@@ -228,7 +228,7 @@ cm_certsave_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 		}
 		PK11_SetPasswordFunc(&cm_pin_read_for_cert_nss_cb);
 		if (entry->cm_cert_token == NULL) {
-			entry->cm_cert_token = talloc_strdup(entry, util_internal_token_name());
+			entry->cm_cert_token = util_internal_token_name(entry);
 		}
 		for (sle = slotlist->head;
 		     ((sle != NULL) && (sle->slot != NULL));
