@@ -579,7 +579,6 @@ main(int argc, const char **argv)
 				pin = NULL;
 			}
 		}
-		free(soptions);
 		/* Add client creds. */
 		if (uid != NULL) {
 			uid = cm_submit_u_url_encode(uid);
@@ -692,7 +691,7 @@ main(int argc, const char **argv)
 	/* Submit the form(s). */
 	hctx = NULL;
 	while (url != NULL) {
-		hctx = cm_submit_h_init(ctx, method, url, params, NULL, NULL,
+		hctx = cm_submit_h_init(ctx, method, url, params, NULL, NULL, NULL,
 					cainfo, capath, sslcert, sslkey, sslpin,
 					cm_submit_h_negotiate_off,
 					cm_submit_h_delegate_off,
