@@ -285,7 +285,7 @@ cm_keygen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 		_exit(CM_SUB_STATUS_ERROR_NO_TOKEN);
 	}
 	if (entry->cm_cert_token == NULL) {
-		entry->cm_cert_token = talloc_strdup(entry, util_internal_token_name());
+		entry->cm_cert_token = util_internal_token_name(entry);
 	}
 	/* Walk the list looking for the requested slot, or the first one if
 	 * none was requested. */
