@@ -165,7 +165,7 @@ cm_keyiread_n_get_keys(struct cm_store_entry *entry, int readwrite)
 	}
 	PK11_SetPasswordFunc(&cm_pin_read_for_cert_nss_cb);
 	if (entry->cm_key_token == NULL) {
-		entry->cm_key_token = talloc_strdup(entry, util_internal_token_name());
+		entry->cm_key_token = util_internal_token_name(entry);
 	}
 	n_tokens = 0;
 	pubkey = NULL;
@@ -557,6 +557,7 @@ cm_keyiread_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 					(name != NULL ? "/" : ""),
 					(name != NULL ? name : ""));
 				status = 0;
+				SECITEM_FreeItem(info, PR_TRUE);
 			} else {
 				cm_log(1, "Error reading public key.\n");
 			}
@@ -599,6 +600,7 @@ cm_keyiread_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 					pubihex,
 					pubhex);
 				status = 0;
+				SECITEM_FreeItem(info, PR_TRUE);
 			}
 		}
 		if (keys->pubkey != NULL) {
